/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
#include <config.h>
#include "gst-package.h"
#include "gst-tool.h"
#include "gst-dialog.h"
#include "gst-filter.h"
#include "gst-service-role.h"
